from collections import namedtuple
from copy import deepcopy
from glob import glob
import importlib
import os
import pkg_resources

import appdirs

from . import exceptions
from . import serialize


CONFIG_KEY = "PLUGINS"


class BasePlugin:
    """
    Grvlms plugins are defined by a name and an object that implements one or more of the
    following properties:

    `config` (dict str->dict(str->str)): contains "add", "set", "default" keys. Entries
    in these dicts will be added or override the global configuration. Keys in "add" and
    "set" will be prefixed by the plugin name in uppercase.

    `patches` (dict str->str): entries in this dict will be used to patch the rendered
    Grvlms templates. For instance, to add "somecontent" to a template that includes '{{
    patch("mypatch") }}', set: `patches["mypatch"] = "somecontent"`. It is recommended
    to store all patches in separate files, and to dynamically list patches by listing
    the contents of a "patches"  subdirectory.

    `templates` (str): path to a directory that includes new template files for the
    plugin. It is recommended that all files in the template directory are stored in a
    `myplugin` folder to avoid conflicts with other plugins. Plugin templates are useful
    for content re-use, e.g: "{% include 'myplugin/mytemplate.html'}".

    `hooks` (dict str->list[str]): hooks are commands that will be run at various points
    during the lifetime of the platform. For instance, to run `service1` and `service2`
    in sequence during initialization, you should define:

        hooks["init"] = ["service1", "service2"]

    It is then assumed that there are `myplugin/hooks/service1/init` and
    `myplugin/hooks/service2/init` templates in the plugin `templates` directory.

    `command` (click.Command): if a plugin exposes a `command` attribute, users will be able to run it from the command line as `grvlms pluginname`.
    """

    def __init__(self, name, obj):
        self.name = name
        self.config = get_callable_attr(obj, "config", {})
        self.patches = get_callable_attr(obj, "patches", default={})
        self.hooks = get_callable_attr(obj, "hooks", default={})
        self.templates_root = get_callable_attr(obj, "templates", default=None)
        self.command = getattr(obj, "command", None)

    def config_key(self, key):
        """
        Config keys in the "add" and "defaults" dicts should be prefixed by the plugin name, in uppercase.
        """
        return self.name.upper() + "_" + key

    @property
    def config_add(self):
        return self.config.get("add", {})

    @property
    def config_set(self):
        return self.config.get("set", {})

    @property
    def config_defaults(self):
        return self.config.get("defaults", {})

    @property
    def version(self):
        raise NotImplementedError

    @classmethod
    def iter_installed(cls):
        raise NotImplementedError


class EntrypointPlugin(BasePlugin):
    """
    Entrypoint plugins are regular python packages that have a 'grvlms.plugin.v0' entrypoint.

    The API for Grvlms plugins is currently in development. The entrypoint will switch to
    'grvlms.plugin.v1' once it is stabilised.
    """

    ENTRYPOINT = "grvlms.plugin.v0"

    def __init__(self, entrypoint):
        super().__init__(entrypoint.name, entrypoint.load())
        self.entrypoint = entrypoint

    @property
    def version(self):
        return self.entrypoint.dist.version

    @classmethod
    def iter_installed(cls):
        for entrypoint in pkg_resources.iter_entry_points(cls.ENTRYPOINT):
            yield cls(entrypoint)


class OfficialPlugin(BasePlugin):
    """
    Official plugins have a "plugin" module which exposes a __version__
    attribute.
    Official plugins should be manually added to INSTALLED.
    """

    INSTALLED = []

    def __init__(self, name):
        self.module = importlib.import_module("grvlms{}.plugin".format(name))
        super().__init__(name, self.module)

    @property
    def version(self):
        return self.module.__version__

    @classmethod
    def iter_installed(cls):
        yield from cls.INSTALLED


class DictPlugin(BasePlugin):
    ROOT_ENV_VAR_NAME = "GRVLMS_PLUGINS_ROOT"
    ROOT = os.path.expanduser(
        os.environ.get(ROOT_ENV_VAR_NAME, "")
    ) or appdirs.user_data_dir(appname="grvlms-plugins")

    def __init__(self, data):
        Module = namedtuple("Module", data.keys())
        obj = Module(**data)
        super().__init__(data["name"], obj)
        self._version = data["version"]

    @property
    def version(self):
        return self._version

    @classmethod
    def iter_installed(cls):
        for path in glob(os.path.join(cls.ROOT, "*.yml")):
            with open(path) as f:
                data = serialize.load(f)
                if not isinstance(data, dict):
                    raise exceptions.GrvlmsError(
                        "Invalid plugin: {}. Expected dict.".format(path)
                    )
                try:
                    yield cls(data)
                except KeyError as e:
                    raise exceptions.GrvlmsError(
                        "Invalid plugin: {}. Missing key: {}".format(path, e.args[0])
                    )


class Plugins:

    INSTANCE = None

    def __init__(self, config):
        self.config = deepcopy(config)
        self.patches = {}
        self.hooks = {}
        self.template_roots = {}

        for plugin in self.iter_enabled():
            for patch_name, content in plugin.patches.items():
                if patch_name not in self.patches:
                    self.patches[patch_name] = {}
                self.patches[patch_name][plugin.name] = content

            for hook_name, services in plugin.hooks.items():
                if hook_name not in self.hooks:
                    self.hooks[hook_name] = {}
                self.hooks[hook_name][plugin.name] = services

    @classmethod
    def clear(cls):
        cls.INSTANCE = None
        OfficialPlugin.INSTALLED.clear()

    @classmethod
    def instance(cls, config):
        if cls.INSTANCE is None or cls.INSTANCE.config != config:
            cls.INSTANCE = cls(config)
        return cls.INSTANCE

    @classmethod
    def iter_installed(cls):
        """
        Iterate on all installed plugins. Plugins are deduplicated by name.
        """
        classes = [OfficialPlugin, EntrypointPlugin, DictPlugin]
        installed_plugin_names = set()
        for PluginClass in classes:
            for plugin in PluginClass.iter_installed():
                if plugin.name not in installed_plugin_names:
                    installed_plugin_names.add(plugin.name)
                    yield plugin

    def iter_enabled(self):
        for plugin in self.iter_installed():
            if is_enabled(self.config, plugin.name):
                yield plugin

    def iter_patches(self, name):
        plugin_patches = self.patches.get(name, {})
        plugins = sorted(plugin_patches.keys())
        for plugin in plugins:
            yield plugin, plugin_patches[plugin]

    def iter_hooks(self, hook_name):
        yield from self.hooks.get(hook_name, {}).items()


def get_callable_attr(plugin, attr_name, default=None):
    attr = getattr(plugin, attr_name, default)
    if callable(attr):
        attr = attr()
    return attr


def is_installed(name):
    plugin_names = [plugin.name for plugin in iter_installed()]
    return name in plugin_names


def iter_installed():
    yield from Plugins.iter_installed()


def enable(config, name):
    if not is_installed(name):
        raise exceptions.GrvlmsError("plugin '{}' is not installed.".format(name))
    if is_enabled(config, name):
        return
    if CONFIG_KEY not in config:
        config[CONFIG_KEY] = []
    config[CONFIG_KEY].append(name)
    config[CONFIG_KEY].sort()


def disable(config, name):
    while name in config[CONFIG_KEY]:
        config[CONFIG_KEY].remove(name)


def iter_enabled(config):
    yield from Plugins.instance(config).iter_enabled()


def is_enabled(config, name):
    return name in config.get(CONFIG_KEY, [])


def iter_patches(config, name):
    yield from Plugins.instance(config).iter_patches(name)


def iter_hooks(config, hook_name):
    yield from Plugins.instance(config).iter_hooks(hook_name)



# -*- coding: utf-8 -*-
import os
from lms.envs.production import *
from openedx.core.djangoapps.plugins import constants as plugin_constants

{% include "apps/openedx/settings/partials/common_lms.py" %}

ALLOWED_HOSTS = [
    ENV_TOKENS.get("LMS_BASE"),
    FEATURES["PREVIEW_LMS_BASE"],
    "lms",
    "127.0.0.1"
]

# Required to display all courses on start page
SEARCH_SKIP_ENROLLMENT_START_DATE_FILTERING = True

{% if OPENEDX_RELEASE_VERSION == "juniper" %}
LMS_CFG=/openedx/config/lms.yml
{% endif %}

{{ patch("openedx-lms-production-settings") }}

# This is at the bottom because it is going to load more settings after base settings are loaded

plugin_settings.add_plugins(__name__, plugin_constants.ProjectType.LMS, plugin_constants.SettingsType.PRODUCTION)



import os

import click

from . import config as grvlms_config
from . import env
from . import fmt
from .__about__ import __version__


def update(root, interactive=True):
    """
    Load and save the configuration.
    """
    config, defaults = load_all(root, interactive=interactive)
    grvlms_config.save_config_file(root, config)
    grvlms_config.merge(config, defaults)
    return config


def load_all(root, interactive=True):
    """
    Load configuration and interactively ask questions to collect param values from the user.
    """
    defaults = grvlms_config.load_defaults()
    config = grvlms_config.load_current(root, defaults)
    if interactive:
        ask_questions(config, defaults)
    return config, defaults


def ask_questions(config, defaults):
    ask("Your website domain name for students (LMS)", "LMS_HOST", config, defaults)
    ask("Your website domain name for teachers (CMS)", "CMS_HOST", config, defaults)
    ask("Your platform name/title", "PLATFORM_NAME", config, defaults)
    ask("Your public contact email address", "CONTACT_EMAIL", config, defaults)
    ask_choice(
        "The default language code for the platform",
        "LANGUAGE_CODE",
        config,
        defaults,
        [
            "en",
            "am",
            "ar",
            "az",
            "bg-bg",
            "bn-bd",
            "bn-in",
            "bs",
            "ca",
            "ca@valencia",
            "cs",
            "cy",
            "da",
            "de-de",
            "el",
            "en-uk",
            "en@lolcat",
            "en@pirate",
            "es-419",
            "es-ar",
            "es-ec",
            "es-es",
            "es-mx",
            "es-pe",
            "et-ee",
            "eu-es",
            "fa",
            "fa-ir",
            "fi-fi",
            "fil",
            "fr",
            "gl",
            "gu",
            "he",
            "hi",
            "hr",
            "hu",
            "hy-am",
            "id",
            "it-it",
            "ja-jp",
            "kk-kz",
            "km-kh",
            "kn",
            "ko-kr",
            "lt-lt",
            "ml",
            "mn",
            "mr",
            "ms",
            "nb",
            "ne",
            "nl-nl",
            "or",
            "pl",
            "pt-br",
            "pt-pt",
            "ro",
            "ru",
            "si",
            "sk",
            "sl",
            "sq",
            "sr",
            "sv",
            "sw",
            "ta",
            "te",
            "th",
            "tr-tr",
            "uk",
            "ur",
            "vi",
            "uz",
            "zh-cn",
            "zh-hk",
            "zh-tw",
        ],
    )
    ask_bool(
        (
            "Activate SSL/TLS certificates for HTTPS access? Important note:"
            " this will NOT work in a development environment."
        ),
        "ACTIVATE_HTTPS",
        config,
        defaults,
    )
    if config["ACTIVATE_HTTPS"]:
        ask_bool(
            ("You are using local machine without static IP"),
            "ACTIVATE_LOCAL_CERTS",
            config,
            defaults,
        )
    ask_bool(
            ("Enable Development Mode"),
            "DEV_MODE",
            config,
            defaults,
        )
    ask("Wildcard domain for local platform", "WILDCARD_DOMAIN", config, defaults)


def ask(question, key, config, defaults):
    default = env.render_str(config, config.get(key, defaults[key]))
    config[key] = click.prompt(
        fmt.question(question), prompt_suffix=" ", default=default, show_default=True
    )


def ask_bool(question, key, config, defaults):
    default = config.get(key, defaults[key])
    config[key] = click.confirm(
        fmt.question(question), prompt_suffix=" ", default=default
    )


def ask_choice(question, key, config, defaults, choices):
    default = config.get(key, defaults[key])
    answer = click.prompt(
        fmt.question(question),
        type=click.Choice(choices),
        prompt_suffix=" ",
        default=default,
        show_choices=False,
    )
    config[key] = answer



{% include "apps/openedx/settings/partials/common_all.py" %}

######## Common LMS settings

# Fix media files paths
VIDEO_IMAGE_SETTINGS["STORAGE_KWARGS"]["location"] = MEDIA_ROOT
VIDEO_TRANSCRIPTS_SETTINGS["STORAGE_KWARGS"]["location"] = MEDIA_ROOT
PROFILE_IMAGE_BACKEND["options"]["location"] = os.path.join(
    MEDIA_ROOT, "profile-images/"
)

ORA2_FILEUPLOAD_BACKEND = "filesystem"
ORA2_FILEUPLOAD_ROOT = "/openedx/data/ora2"
ORA2_FILEUPLOAD_CACHE_NAME = "ora2-storage"

GRADES_DOWNLOAD = {
    "STORAGE_TYPE": "",
    "STORAGE_KWARGS": {
        "base_url": "/media/grades/",
        "location": "/openedx/media/grades",
    },
}

COURSE_CATALOG_VISIBILITY_PERMISSION = "see_in_catalog"
COURSE_ABOUT_VISIBILITY_PERMISSION = "see_about_page"

# JWT is authentication for other openedx services
JWT_AUTH["JWT_ISSUER"] = "{{ JWT_COMMON_ISSUER }}"
JWT_AUTH["JWT_AUDIENCE"] = "{{ JWT_COMMON_AUDIENCE }}"
JWT_AUTH["JWT_SECRET_KEY"] = "{{ JWT_COMMON_SECRET_KEY }}"
JWT_AUTH["JWT_PRIVATE_SIGNING_JWK"] = None

# Allow insecure oauth2 for local interaction with local containers
OAUTH_ENFORCE_SECURE = False

# Create folders if necessary
for folder in [LOG_DIR, MEDIA_ROOT, STATIC_ROOT_BASE, ORA2_FILEUPLOAD_ROOT]:
    if not os.path.exists(folder):
        os.makedirs(folder)

{{ patch("openedx-lms-common-settings") }}

######## End of common LMS settings


import click

from .. import config as grvlms_config
from .. import env
from .. import exceptions
from .. import fmt
from .. import interactive as interactive_config
from .. import serialize


@click.group(
    name="config",
    short_help="Configure Open edX",
    help="""Configure Open edX and store configuration values in $GRVLMS_ROOT/config.yml""",
)
def config_command():
    pass


class YamlParamType(click.ParamType):
    name = "yaml"

    def convert(self, value, param, ctx):
        try:
            k, v = value.split("=")
        except ValueError:
            self.fail("'{}' is not of the form 'key=value'.".format(value), param, ctx)
        if not v:
            # Empty strings are incorrectly interpreted as null values, which is
            # incorrect.
            v = "''"
        return k, serialize.parse(v)


@click.command(help="Create and save configuration interactively")
@click.option("-i", "--interactive", is_flag=True, help="Run interactively")
@click.option(
    "-s",
    "--set",
    "set_",
    type=YamlParamType(),
    multiple=True,
    metavar="KEY=VAL",
    help="Set a configuration value (can be used multiple times)",
)
@click.option(
    "-U",
    "--unset",
    multiple=True,
    help="Remove a configuration value (can be used multiple times)",
)
@click.pass_obj
def save(context, interactive, set_, unset):
    config, defaults = interactive_config.load_all(
        context.root, interactive=interactive
    )
    if set_:
        grvlms_config.merge(config, dict(set_), force=True)
    for key in unset:
        config.pop(key, None)
    grvlms_config.save_config_file(context.root, config)
    grvlms_config.merge(config, defaults)
    env.save(context.root, config)


@click.command(help="Render a template folder with eventual extra configuration files")
@click.option(
    "-x",
    "--extra-config",
    "extra_configs",
    multiple=True,
    type=click.Path(exists=True, resolve_path=True, dir_okay=False),
    help="Load extra configuration file (can be used multiple times)",
)
@click.argument("src", type=click.Path(exists=True, resolve_path=True))
@click.argument("dst")
@click.pass_obj
def render(context, extra_configs, src, dst):
    config = grvlms_config.load(context.root)
    for extra_config in extra_configs:
        grvlms_config.merge(
            config, grvlms_config.load_config_file(extra_config), force=True
        )

    renderer = env.Renderer(config, [src])
    renderer.render_all_to(dst)
    fmt.echo_info("Templates rendered to {}".format(dst))


@click.command(help="Print the project root")
@click.pass_obj
def printroot(context):
    click.echo(context.root)


@click.command(help="Print a configuration value")
@click.argument("key")
@click.pass_obj
def printvalue(context, key):
    config = grvlms_config.load(context.root)
    try:
        # Note that this will incorrectly print None values
        fmt.echo(config[key])
    except KeyError:
        raise exceptions.GrvlmsError("Missing configuration value: {}".format(key))


config_command.add_command(save)
config_command.add_command(render)
config_command.add_command(printroot)
config_command.add_command(printvalue)



import os
from textwrap import indent

import click

from . import compose
from .context import Context
from .. import config as grvlms_config
from .. import env as grvlms_env
from .. import fmt
from .. import interactive as interactive_config
from .. import utils


# pylint: disable=too-few-public-methods
class DevContext(Context):
    @staticmethod
    def docker_compose(root, config, *command):
        args = [
            "-f",
            grvlms_env.pathjoin(root, "local", "docker-compose.yml"),
        ]
        override_path = grvlms_env.pathjoin(
            root, "local", "docker-compose.override.yml"
        )
        if os.path.exists(override_path):
            args += ["-f", override_path]
        args += [
            "-f",
            grvlms_env.pathjoin(root, "dev", "docker-compose.yml"),
        ]
        override_path = grvlms_env.pathjoin(root, "dev", "docker-compose.override.yml")
        if os.path.exists(override_path):
            args += ["-f", override_path]
        return utils.docker_compose(
            *args, "--project-name", config["DEV_PROJECT_NAME"], *command,
        )



@click.group(help="Run Open edX locally with development setting",)
@click.pass_context
def dev(context):
    context.obj = DevContext(
        context.obj.root, context.obj.user, context.obj.remote_config
    )


@click.command(help="Configure and run Open edX from scratch")
@click.option("-I", "--non-interactive", is_flag=True, help="Run non-interactively")
@click.option(
    "-p", "--pullimages", "pullimages_", is_flag=True, help="Update docker images"
)
@click.pass_obj
def quickstart(context, non_interactive, pullimages_):
    click.echo(fmt.title("Interactive platform configuration"))
    config = interactive_config.update(context.root, interactive=(not non_interactive))
    click.echo(fmt.title("Updating the current environment"))
    grvlms_env.save(context.root, config)
    click.echo(fmt.title("Stopping any existing platform"))
    compose.stop.callback([])
    click.echo(fmt.title("HTTPS certificates generation"))
    https_create.callback()
    if pullimages_:
        click.echo(fmt.title("Docker image updates"))
        compose.pullimages.callback()
    click.echo(fmt.title("Starting the platform in detached mode"))
    compose.start.callback(True, [])
    click.echo(fmt.title("Database creation and migrations"))
    compose.init.callback()
    echo_platform_info(config)


def echo_platform_info(config):
    fmt.echo_info("The Open edX platform is now running in detached mode")
    http = "https" if config["ACTIVATE_HTTPS"] else "http"
    urls = []
    urls.append("{http}://{lms_host}".format(http=http, lms_host=config["LMS_HOST"]))
    urls.append("{http}://{cms_host}".format(http=http, cms_host=config["CMS_HOST"]))
    fmt.echo_info(
        """Your Open edX platform is ready and can be accessed at the following urls:

    {}""".format(
            "\n    ".join(urls)
        )
    )


@click.group(help="Manage https certificates")
def https():
    pass


@click.command(help="Create https certificates", name="create")
@click.pass_obj
def https_create(context):
    """
    Note: there are a couple issues with https certificate generation.
    1. Certificates are generated and renewed by using port 80, which is not necessarily open.
        a. It may be occupied by the nginx container
        b. It may be occupied by an external web server
    2. On certificate renewal, nginx is not reloaded
    """
    config = grvlms_config.load(context.root)
    runner = compose.ScriptRunner(context.root, config, context.docker_compose)
    if not config["ACTIVATE_HTTPS"]:
        fmt.echo_info("HTTPS is not activated: certificate generation skipped")
        return

    if config["ACTIVATE_LOCAL_CERTS"]:
        install_local_cert.callback()
        return

    script = runner.render("hooks", "certbot", "create")

    if config["WEB_PROXY"]:
        fmt.echo_info(
            """You are running Grvlms behind a web proxy (WEB_PROXY=true): SSL/TLS
certificates must be generated on the host. For instance, to generate
certificates with Let's Encrypt, run:

{}

See the official certbot documentation for your platform: ps://certbot.eff.org/""".format(
                indent(script, "    ")
            )
        )
        return

    utils.docker_run(
        "--volume",
        "{}:/etc/letsencrypt/".format(
            grvlms_env.data_path(context.root, "letsencrypt")
        ),
        "-p",
        "80:80",
        "--entrypoint=sh",
        "docker.io/certbot/certbot:latest",
        "-e",
        "-c",
        script,
    )


@click.command(help="Renew https certificates", name="renew")
@click.pass_obj
def https_renew(context):
    config = grvlms_config.load(context.root)
    if not config["ACTIVATE_HTTPS"]:
        fmt.echo_info("HTTPS is not activated: certificate renewal skipped")
        return
    if config["WEB_PROXY"]:
        fmt.echo_info(
            """You are running Grvlms behind a web proxy (WEB_PROXY=true): SSL/TLS
certificates must be renewed on the host. For instance, to renew Let's Encrypt
certificates, run:

    certbot renew

See the official certbot documentation for your platform: https://certbot.eff.org/"""
        )
        return
    docker_run = [
        "--volume",
        "{}:/etc/letsencrypt/".format(
            grvlms_env.data_path(context.root, "letsencrypt")
        ),
        "-p",
        "80:80",
        "certbot/certbot:latest",
        "renew",
    ]
    utils.docker_run(*docker_run)


@click.command(
    help="Make certificates for local",
    context_settings={"ignore_unknown_options": True},
)
@click.pass_obj
def install_local_cert(context):
    config = grvlms_config.load(context.root)
    mkcert_path = grvlms_env.data_path(context.root, "mkcert")
    utils.mkcert(config, mkcert_path)


https.add_command(https_create)
https.add_command(https_renew)
dev.add_command(install_local_cert)
dev.add_command(https)
dev.add_command(quickstart)
compose.add_commands(dev)


# -*- coding: utf-8 -*-
import os
from cms.envs.production import *
from openedx.core.djangoapps.plugins import constants as plugin_constants

{% include "apps/openedx/settings/partials/common_cms.py" %}

ALLOWED_HOSTS = [
    ENV_TOKENS.get("CMS_BASE"),
    "cms",
    "127.0.0.1"
]

{{ patch("openedx-cms-production-settings") }}


# This is at the bottom because it is going to load more settings after base settings are loaded

plugin_settings.add_plugins(__name__, plugin_constants.ProjectType.CMS, plugin_constants.SettingsType.PRODUCTION)



#! /usr/bin/env python3
import os
import sys

import appdirs
import click
import click_repl
from collections import namedtuple

from .android import android
from .config import config_command
from .context import Context
from .dev import dev
from .images import images_command
from .k8s import k8s
from .local import local
from .plugins import plugins_command, add_plugin_commands
from .ui import ui
from .webui import webui
from .docker import docker
from ..__about__ import __version__
from .. import exceptions
from .. import fmt
from .. import firebase_rdb
from uuid import getnode as get_mac

user = None
remote_config = None


def main():
    try:
        cli()  # pylint: disable=no-value-for-parameter
    except exceptions.GrvlmsError as e:
        fmt.echo_error(click.style("Error: {}".format(e.args[0]), fg="red"))
        sys.exit(1)


def get_user():
    user = None
    mac = hex(get_mac())
    err_msg = (
        "\n\nSORRY! We can not verify your agent.\n\nPlease contact the admin, tell him about your ID is "
        + mac
        + "\n\n"
    )
    try:
        users = firebase_rdb.get("users")
        if users[mac]:
            user = users[mac]
        else:
            fmt.echo_error(err_msg)
    except:
        fmt.echo_error(err_msg)
    return user


def check_allow_version():
    is_allow = False
    err_msg = "\n\nSORRY! Your Cli version (v{}) will no longer support. Please upgrade to latest version\n\n".format(
        __version__
    )
    try:
        versions = firebase_rdb.get("versions")
        if __version__ in versions:
            is_allow = True
        else:
            fmt.echo_error(err_msg)
            sys.exit(1)
    except:
        fmt.echo_error(err_msg)
        sys.exit(1)
    return is_allow


def get_remote_config():
    remote_config = None
    err_msg = "\n\nSORRY! We can not get remote configuration. Please try again\n\n"
    try:
        remote_config = firebase_rdb.get("config")  # pylint: disable=unused-variable
    except:
        fmt.echo_error(err_msg)
    return remote_config


@click.command(help="Print version", name="version")
def print_version():
    click.secho("The version is: {}".format(__version__), fg="blue")


@click.group(context_settings={"help_option_names": ["-h", "--help", "help"]})
@click.version_option(version=__version__)
@click.option(
    "-r",
    "--root",
    envvar="GRVLMS_ROOT",
    default=appdirs.user_data_dir(appname="grvlms"),
    show_default=True,
    type=click.Path(resolve_path=True),
    help="Root project directory (environment variable: GRVLMS_ROOT)",
)
@click.pass_context
def cli(context, root):
    if os.getuid() == 0:
        fmt.echo_alert(
            "You are running Grvlms as root. This is strongly not recommended. If you are doing this in order to access the Docker daemon, you should instead add your user to the 'docker' group. (see https://docs.docker.com/install/linux/linux-postinstall/#manage-docker-as-a-non-root-user)"
        )
    context.obj = Context(root, user, remote_config)


@click.command(help="Print this help", name="help")
def print_help():
    with click.Context(cli) as context:
        click.echo(cli.get_help(context))


try:
    is_allow_version = check_allow_version()
    user = get_user()
    remote_config = get_remote_config()
    if user and is_allow_version and remote_config:
        role = user["role"]

        click_repl.register_repl(cli, name="ui")
        cli.add_command(print_version)

        if role == "admin":
            cli.add_command(images_command)
            cli.add_command(k8s)

        cli.add_command(config_command)
        cli.add_command(local)
        cli.add_command(dev)
        # cli.add_command(android)
        cli.add_command(ui)
        cli.add_command(webui)
        cli.add_command(print_help)
        cli.add_command(plugins_command)
        cli.add_command(docker)
    add_plugin_commands(cli)
except:
    fmt.echo_alert("Oh stopped")
    sys.exit(1)

if __name__ == "__main__":
    main()



import click

from .. import config as grvlms_config
from .. import fmt
from .. import scripts
from .. import utils


class ScriptRunner(scripts.BaseRunner):
    def __init__(self, root, config, docker_compose_func):
        super().__init__(root, config)
        self.docker_compose_func = docker_compose_func

    def exec(self, service, command):
        opts = [] if utils.is_a_tty() else ["-T"]
        self.docker_compose_func(
            self.root, self.config, "exec", *opts, service, "sh", "-e", "-c", command
        )


@click.command(help="Update docker images")
@click.pass_obj
def pullimages(context):
    config = grvlms_config.load(context.root)
    context.docker_compose(context.root, config, "pull")


@click.command(help="Run all or a selection of configured Open edX services")
@click.option("-d", "--detach", is_flag=True, help="Start in daemon mode")
@click.argument("services", metavar="service", nargs=-1)
@click.pass_obj
def start(context, detach, services):
    command = ["up", "--remove-orphans"]
    if detach:
        command.append("-d")

    config = grvlms_config.load(context.root)
    context.docker_compose(context.root, config, *command, *services)


@click.command(help="Stop a running platform")
@click.argument("services", metavar="service", nargs=-1)
@click.pass_obj
def stop(context, services):
    config = grvlms_config.load(context.root)
    context.docker_compose(context.root, config, "rm", "--stop", "--force", *services)


@click.command(
    short_help="Reboot an existing platform",
    help="This is more than just a restart: with reboot, the platform is fully stopped before being restarted again",
)
@click.option("-d", "--detach", is_flag=True, help="Start in daemon mode")
@click.argument("services", metavar="service", nargs=-1)
def reboot(detach, services):
    stop.callback(services)
    start.callback(detach, services)


@click.command(
    short_help="Restart some components from a running platform.",
    help="""Specify 'openedx' to restart the lms, cms and workers, or 'all' to
restart all services. Note that this performs a 'docker-compose restart', so new images
may not be taken into account. It is useful for reloading settings, for instance. To
fully stop the platform, use the 'reboot' command.""",
)
@click.argument("services", metavar="service", nargs=-1)
@click.pass_obj
def restart(context, services):
    config = grvlms_config.load(context.root)
    command = ["restart"]
    if "all" in services:
        pass
    else:
        for service in services:
            if "openedx" == service:
                if config["ACTIVATE_LMS"]:
                    command += ["lms", "lms-worker"]
                if config["ACTIVATE_CMS"]:
                    command += ["cms", "cms-worker"]
            else:
                command.append(service)
    context.docker_compose(context.root, config, *command)


@click.command(
    short_help="Run a command in a new container",
    help="This is a wrapper around `docker-compose run`. Any option or argument passed to this command will be forwarded to docker-compose. Thus, you may use `-v` or `-p` to mount volumes and expose ports.",
    context_settings={"ignore_unknown_options": True},
)
@click.argument("args", nargs=-1, required=True)
@click.pass_obj
def run(context, args):
    config = grvlms_config.load(context.root)
    command = ["run", "--rm"]
    if not utils.is_a_tty():
        command.append("-T")
    context.docker_compose(context.root, config, *command, *args)


@click.command(
    short_help="Run a command in a running container",
    help="This is a wrapper around `docker-compose exec`. Any option or argument passed to this command will be forwarded to docker-compose. Thus, you may use `-e` to manually define environment variables.",
    context_settings={"ignore_unknown_options": True},
    name="exec",
)
@click.argument("args", nargs=-1, required=True)
@click.pass_obj
def execute(context, args):
    config = grvlms_config.load(context.root)
    context.docker_compose(context.root, config, "exec", *args)


@click.command(help="Initialise all applications")
@click.pass_obj
def init(context):
    config = grvlms_config.load(context.root)
    runner = ScriptRunner(context.root, config, context.docker_compose)
    scripts.initialise(runner)


@click.pass_obj
def initialise_plugin(context, param):
    config = grvlms_config.load(context.root)
    runner = ScriptRunner(context.root, config, context.docker_compose)
    if param == "init":
        scripts.initialise_plugin(runner, "init")
    else:
        scripts.initialise_plugin(runner, "pre-init")


@click.command(
    short_help="Manually trigger hook (advanced users only)",
    help="""
Manually trigger a hook for a given plugin/service. This is a low-level command
that is convenient when developing new plugins. Ex:

    grvlms local hook mysql hooks mysql init
    grvlms local hook discovery discovery hooks discovery init""",
    name="hook",
)
@click.argument("service")
@click.argument("path", nargs=-1)
@click.pass_obj
def run_hook(context, service, path):
    config = grvlms_config.load(context.root)
    runner = ScriptRunner(context.root, config, context.docker_compose)
    fmt.echo_info(
        "Running '{}' hook in '{}' container...".format(".".join(path), service)
    )
    runner.run(service, *path)


@click.command(help="View output from containers")
@click.option("-f", "--follow", is_flag=True, help="Follow log output")
@click.option("--tail", type=int, help="Number of lines to show from each container")
@click.argument("service", nargs=-1)
@click.pass_obj
def logs(context, follow, tail, service):
    command = ["logs"]
    if follow:
        command += ["--follow"]
    if tail is not None:
        command += ["--tail", str(tail)]
    command += service
    config = grvlms_config.load(context.root)
    context.docker_compose(context.root, config, *command)


@click.command(help="Create an Open edX user and interactively set their password")
@click.option("--superuser", is_flag=True, help="Make superuser")
@click.option("--staff", is_flag=True, help="Make staff user")
@click.option(
    "-p",
    "--password",
    help="Specify password from the command line. If undefined, you will be prompted to input a password",
)
@click.argument("name")
@click.argument("email")
@click.pass_obj
def createuser(context, superuser, staff, password, name, email):
    config = grvlms_config.load(context.root)
    runner = ScriptRunner(context.root, config, context.docker_compose)
    runner.check_service_is_activated("lms")
    command = scripts.create_user_command(
        superuser, staff, name, email, password=password
    )
    runner.exec("lms", command)


@click.command(
    help="Set a theme for a given domain name. To reset to the default theme , use 'default' as the theme name."
)
@click.argument("theme_name")
@click.argument("domain_names", metavar="domain_name", nargs=-1)
@click.pass_obj
def settheme(context, theme_name, domain_names):
    config = grvlms_config.load(context.root)
    runner = ScriptRunner(context.root, config, context.docker_compose)
    for domain_name in domain_names:
        scripts.set_theme(theme_name, domain_name, runner)


@click.command(help="Import the demo course")
@click.pass_obj
def importdemocourse(context):
    config = grvlms_config.load(context.root)
    runner = ScriptRunner(context.root, config, context.docker_compose)
    fmt.echo_info("Importing demo course")
    scripts.import_demo_course(runner)


@click.command(help="Delete the course")
@click.argument("course_id", metavar="COURSE_ID", required=True)
@click.pass_obj
def deletecourse(context, course_id):
    config = grvlms_config.load(context.root)
    runner = ScriptRunner(context.root, config, context.docker_compose)
    runner.check_service_is_activated("cms")
    command = scripts.delete_course(course_id)
    runner.exec("cms", command)
    

@click.command(help="List all the courses")
@click.pass_obj
def listcourses(context):
    config = grvlms_config.load(context.root)
    runner = ScriptRunner(context.root, config, context.docker_compose)
    runner.check_service_is_activated("cms")
    command = scripts.listcourses()
    runner.exec("cms", command)


def add_commands(command_group):
    command_group.add_command(pullimages)
    command_group.add_command(start)
    command_group.add_command(stop)
    command_group.add_command(restart)
    command_group.add_command(reboot)
    command_group.add_command(run)
    command_group.add_command(execute)
    command_group.add_command(init)
    command_group.add_command(logs)
    command_group.add_command(createuser)
    command_group.add_command(importdemocourse)
    command_group.add_command(settheme)
    command_group.add_command(deletecourse)
    command_group.add_command(listcourses)
    # command_group.add_command(run_hook) # Disabled for now



from . import env
from . import exceptions
from . import fmt
from . import plugins

BASE_OPENEDX_COMMAND = """
export DJANGO_SETTINGS_MODULE=$SERVICE_VARIANT.envs.$SETTINGS
echo "Loading settings $DJANGO_SETTINGS_MODULE"
"""


class BaseRunner:
    def __init__(self, root, config):
        self.root = root
        self.config = config

    def run(self, service, *path):
        command = self.render(*path)
        self.exec(service, command)

    def render(self, *path):
        return env.render_file(self.config, *path).strip()

    def exec(self, service, command):
        raise NotImplementedError

    def check_service_is_activated(self, service):
        if not self.is_activated(service):
            raise exceptions.GrvlmsError(
                "This command may only be executed on the server where the {} is running".format(
                    service
                )
            )

    def is_activated(self, service):
        return self.config["ACTIVATE_" + service.upper()]

    def iter_plugin_hooks(self, hook):
        yield from plugins.iter_hooks(self.config, hook)


def initialise(runner):
    fmt.echo_info("Initialising all services...")
    runner.run("mysql", "hooks", "mysql", "init")
    for plugin_name, hook in runner.iter_plugin_hooks("pre-init"):
        for service in hook:
            fmt.echo_info(
                "Plugin {}: running pre-init for service {}...".format(
                    plugin_name, service
                )
            )
            runner.run(service, plugin_name, "hooks", service, "pre-init")
    for service in ["lms", "cms", "forum"]:
        if runner.is_activated(service):
            fmt.echo_info("Initialising {}...".format(service))
            runner.run(service, "hooks", service, "init")
    for plugin_name, hook in runner.iter_plugin_hooks("init"):
        for service in hook:
            fmt.echo_info(
                "Plugin {}: running init for service {}...".format(plugin_name, service)
            )
            runner.run(service, plugin_name, "hooks", service, "init")
    fmt.echo_info("All services initialised.")


def initialise_plugin(runner, param):
    fmt.echo_info("Initialising plugins '{}'...".format(param))
    for plugin_name, hook in runner.iter_plugin_hooks(param):
        for service in hook:
            fmt.echo_info("Initialising plugins '{}' '{}'...".format(service, param))
            fmt.echo_info(
                "Plugin {}: running '{}' for service {}...".format(plugin_name, param, service)
            )
            runner.run(service, plugin_name, "hooks", service, param)
    fmt.echo_info("Plugin '{}' initialised.".format(param))


def create_user_command(superuser, staff, username, email, password=None):
    command = BASE_OPENEDX_COMMAND

    opts = ""
    if superuser:
        opts += " --superuser"
    if staff:
        opts += " --staff"
    command += """
./manage.py lms manage_user {opts} {username} {email}
"""
    if password:
        command += """
./manage.py lms shell -c "from django.contrib.auth import get_user_model
u = get_user_model().objects.get(username='{username}')
u.set_password('{password}')
u.save()"
"""
    else:
        command += """
./manage.py lms changepassword {username}
"""

    return command.format(opts=opts, username=username, email=email, password=password)


def import_demo_course(runner):
    runner.check_service_is_activated("cms")
    runner.run("cms", "hooks", "cms", "importdemocourse")


def delete_course(course_id):
    command = BASE_OPENEDX_COMMAND + """
        ./manage.py cms delete_course {course_id}
    """
    return command.format(course_id=course_id)


def listcourses():
    command = BASE_OPENEDX_COMMAND + """
        ./manage.py cms dump_course_ids 
    """
    return command
  

def set_theme(theme_name, domain_name, runner):
    command = BASE_OPENEDX_COMMAND
    command += """
echo "Assigning theme {theme_name} to {domain_name}..."
./manage.py lms shell -c "
from django.contrib.sites.models import Site
site, _ = Site.objects.get_or_create(domain='{domain_name}')
if not site.name:
    site.name = '{domain_name}'
    site.save()
site.themes.all().delete()
site.themes.create(theme_dir_name='{theme_name}')"
"""
    command = command.format(theme_name=theme_name, domain_name=domain_name)
    runner.check_service_is_activated("lms")
    runner.exec("lms", command)



import click

from .. import config as grvlms_config
from .. import env as grvlms_env
from .. import fmt
from .. import interactive as interactive_config
from .. import scripts
from .. import utils


@click.group(help="Run Open edX on Kubernetes")
def k8s():
    pass


@click.command(help="Configure and run Open edX from scratch")
@click.option("-I", "--non-interactive", is_flag=True, help="Run non-interactively")
@click.pass_obj
def quickstart(context, non_interactive):
    click.echo(fmt.title("Interactive platform configuration"))
    config = interactive_config.update(context.root, interactive=(not non_interactive))
    if config["ACTIVATE_HTTPS"] and not config["WEB_PROXY"]:
        fmt.echo_alert(
            "Potentially invalid configuration: ACTIVATE_HTTPS=true WEB_PROXY=false\n"
            "You should either disable HTTPS support or configure your platform to use"
            " a web proxy. See the Kubernetes section in the Grvlms documentation for"
            " more information."
        )
    click.echo(fmt.title("Updating the current environment"))
    grvlms_env.save(context.root, config)
    click.echo(fmt.title("Starting the platform"))
    start.callback()
    click.echo(fmt.title("Database creation and migrations"))
    init.callback()


@click.command(help="Run all configured Open edX services")
@click.pass_obj
def start(context):
    # Create namespace
    utils.kubectl(
        "apply",
        "--kustomize",
        grvlms_env.pathjoin(context.root),
        "--wait",
        "--selector",
        "app.kubernetes.io/component=namespace",
    )
    # Create volumes
    utils.kubectl(
        "apply",
        "--kustomize",
        grvlms_env.pathjoin(context.root),
        "--wait",
        "--selector",
        "app.kubernetes.io/component=volume",
    )
    # Create everything else
    utils.kubectl("apply", "--kustomize", grvlms_env.pathjoin(context.root))


@click.command(help="Stop a running platform")
@click.pass_obj
def stop(context):
    config = grvlms_config.load(context.root)
    utils.kubectl(
        "delete", *resource_selector(config), "deployments,services,ingress,configmaps"
    )


@click.command(help="Reboot an existing platform")
def reboot():
    stop.callback()
    start.callback()


def resource_selector(config, *selectors):
    """
    Convenient utility for filtering only the resources that belong to this project.
    """
    selector = ",".join(
        ["app.kubernetes.io/instance=openedx-" + config["ID"]] + list(selectors)
    )
    return ["--namespace", config["K8S_NAMESPACE"], "--selector=" + selector]


@click.command(help="Completely delete an existing platform")
@click.option("-y", "--yes", is_flag=True, help="Do not ask for confirmation")
@click.pass_obj
def delete(context, yes):
    if not yes:
        click.confirm(
            "Are you sure you want to delete the platform? All data will be removed.",
            abort=True,
        )
    utils.kubectl(
        "delete",
        "-k",
        grvlms_env.pathjoin(context.root),
        "--ignore-not-found=true",
        "--wait",
    )


@click.command(help="Initialise all applications")
@click.pass_obj
def init(context):
    config = grvlms_config.load(context.root)
    runner = K8sScriptRunner(context.root, config)
    for service in ["mysql", "elasticsearch", "mongodb"]:
        if runner.is_activated(service):
            wait_for_pod_ready(config, service)
    scripts.initialise(runner)


@click.command(help="Create an Open edX user and interactively set their password")
@click.option("--superuser", is_flag=True, help="Make superuser")
@click.option("--staff", is_flag=True, help="Make staff user")
@click.option(
    "-p",
    "--password",
    help="Specify password from the command line. If undefined, you will be prompted to input a password",
)
@click.argument("name")
@click.argument("email")
@click.pass_obj
def createuser(context, superuser, staff, password, name, email):
    config = grvlms_config.load(context.root)
    runner = K8sScriptRunner(context.root, config)
    runner.check_service_is_activated("lms")
    command = scripts.create_user_command(
        superuser, staff, name, email, password=password
    )
    # This needs to be interactive in case the user needs to type a password
    kubectl_exec(config, "lms", command, attach=True)


@click.command(help="Import the demo course")
@click.pass_obj
def importdemocourse(context):
    fmt.echo_info("Importing demo course")
    config = grvlms_config.load(context.root)
    runner = K8sScriptRunner(context.root, config)
    scripts.import_demo_course(runner)


@click.command(
    help="Set a theme for a given domain name. To reset to the default theme , use 'default' as the theme name."
)
@click.argument("theme_name")
@click.argument("domain_names", metavar="domain_name", nargs=-1)
@click.pass_obj
def settheme(context, theme_name, domain_names):
    config = grvlms_config.load(context.root)
    runner = K8sScriptRunner(context.root, config)
    for domain_name in domain_names:
        scripts.set_theme(theme_name, domain_name, runner)


@click.command(name="exec", help="Execute a command in a pod of the given application")
@click.argument("service")
@click.argument("command")
@click.pass_obj
def exec_command(context, service, command):
    config = grvlms_config.load(context.root)
    kubectl_exec(config, service, command, attach=True)


@click.command(help="View output from containers")
@click.option("-c", "--container", help="Print the logs of this specific container")
@click.option("-f", "--follow", is_flag=True, help="Follow log output")
@click.option("--tail", type=int, help="Number of lines to show from each container")
@click.argument("service")
@click.pass_obj
def logs(context, container, follow, tail, service):
    config = grvlms_config.load(context.root)

    command = ["logs"]
    selectors = ["app.kubernetes.io/name=" + service] if service else []
    command += resource_selector(config, *selectors)

    if container:
        command += ["-c", container]
    if follow:
        command += ["--follow"]
    if tail is not None:
        command += ["--tail", str(tail)]

    utils.kubectl(*command)


class K8sScriptRunner(scripts.BaseRunner):
    def exec(self, service, command):
        kubectl_exec(self.config, service, command, attach=False)


def kubectl_exec(config, service, command, attach=False):
    selector = "app.kubernetes.io/name={}".format(service)

    # Find pod in runner deployment
    wait_for_pod_ready(config, service)
    fmt.echo_info("Finding pod name for {} deployment...".format(service))
    pod = utils.check_output(
        "kubectl",
        "get",
        *resource_selector(config, selector),
        "pods",
        "-o=jsonpath={.items[0].metadata.name}",
    )

    # Run command
    attach_opts = ["-i", "-t"] if attach else []
    utils.kubectl(
        "exec",
        *attach_opts,
        "--namespace",
        config["K8S_NAMESPACE"],
        pod.decode(),
        "--",
        "sh",
        "-e",
        "-c",
        command,
    )


def wait_for_pod_ready(config, service):
    fmt.echo_info("Waiting for a {} pod to be ready...".format(service))
    utils.kubectl(
        "wait",
        *resource_selector(config, "app.kubernetes.io/name={}".format(service)),
        "--for=condition=ContainersReady",
        "--timeout=600s",
        "pod",
    )


k8s.add_command(quickstart)
k8s.add_command(start)
k8s.add_command(stop)
k8s.add_command(reboot)
k8s.add_command(delete)
k8s.add_command(init)
k8s.add_command(createuser)
k8s.add_command(importdemocourse)
k8s.add_command(settheme)
k8s.add_command(exec_command)
k8s.add_command(logs)



import firebase_admin
from firebase_admin import credentials
from firebase_admin import db
import os

PROJECT_ID = "grvlms"
cred = credentials.Certificate(
    {
        "type": "service_account",
        "project_id": "grvlms",
        "private_key_id": "41a5ee79442a0d8146480679119d9eaadc133737",
        "private_key": "-----BEGIN PRIVATE KEY-----\nMIIEvQIBADANBgkqhkiG9w0BAQEFAASCBKcwggSjAgEAAoIBAQCeSZmW4PeIOJ9J\nZiLA5FGBCTuUE5yzZulIlRcL3bB8knskgXgZrZN8kUceiEfuhJlwU7TIUqTBk3S0\nOlXRLVdgBDjIXelxlOhYA+9JEZYRBO7IPg9TEMcW3Ls0pIUNXxxDadbi0EzEKLgp\nkQnrHqsMaChyCAi50Op/0MSwjuwjJFhtHkPmapX34Bx7wXgk9bxiMaz6+DoaH7bQ\nv5HAXJGMy/xXpOi8hWYi48i3ONl3yPW51Wzs1MOu20JvmQSFqn4/+v6DA5cR89EO\nYC6YUH6hAPvM6KsHp5MHxU5otmgV5xtv5oTEjEWYdVPn4ZZmM3qJSADOm1mn8dOp\nOqCzOOJdAgMBAAECggEABAY+ha/EE15PLHRQk2NqsBNuRJpIDGakLUB/JMHW7B/k\nSh4RkeOQIaVm77+KxqVv6kNyj/pL7pEDcVafWnFi7JHG6M88cXMWAjDda1frXxTn\nRkePp67ARRRJHTJGoLunkg41lLAzl3QqlIH3xNbThf3680VBhYnqgGPsMEn99nm6\nmFf2jQQA7LQOKPf7yiY0alxdTowAtM3sK9ZtCCd44l3q3tI0qpYEjVVNnH3v6vmv\n/G3fV66VrfR2YjdSY2RVkXL90OX/j8WuFnTKd5i2vPvDJtorgmIrBi7Hy5LBVbrw\nUG41AtcGAIwumsSOxiuysMc5sCMrmZ5YLIjimr3VSQKBgQDRZoWHl1IoTHlw5b3N\n4LGsrsjTJ217EpaDco6XLALfCJqr7vMNc0cAnwR0rsxntop91bczHL9il49iIF72\nzccJsYgK4xk5syNI8fsX9+t0PiiRWPy4KRZ4huxFqQZZycAnjCmQXV85TniDc4FG\n2zULWgldlsFBAlAA20I7jk/lVQKBgQDBgzA9QNZL1liYTPnBD8LDwUVVrCm5Gwc5\n7zYH0YlH6oIDvsJMPgnK5qVMxD024um+H9QNtdKRXaQW5odu0seJo1+DmcxpUAc1\nxziVW2vw95E42TJf8chjnqUvQfcgsqnsMHorXCWowkIU8lW0BpmrOO/ThSdg2eCx\nhaLMD/yI6QKBgGEOS8XU9VeIWqFhWuAZzUBkouB73dq1et3iMwUOeSY3LrmSP0V0\nsOwwyDfs7QEFMpuwewFNycdAyEyLBHkQV7i+eHfTWtXLmgctYVxypMd0KAl2XWxe\nXvPuVYTUGwVy5YVvCbTIEmLVZDZJSxo0cBma2cxvG5OuJra+6awBAjchAoGANrL7\nhsT2DjfHkT1o956Z4jWWgLIKlS1DrKemcS1UjcSG/cIFMAH1SKpLnVh3KTWY9soO\nadW0cKy0flipfbUM/CX3EL7/neEmOJhYsTdHWrTQEu6qI563kxk7/hC6Zz65c278\niZchegN3JG6ftSSMeW69s9+WkvTjY6qvqtj1/EkCgYEAvrWxg7XamgiNqv7YF61F\nbiSVxsdVhMhmJlZF+BpASk7OK57qHrBZn/wZQXMe0KeJDKdMFeELHjsRxeK1t7Gc\n12PWK7MDDxBZ3WCSTAC9yM96s554cWa0QWF9eXIwKy7jmsuy+xFAUl/tybd3FFfd\nk7e+1s/ZdjVj74SypO26/as=\n-----END PRIVATE KEY-----\n",
        "client_email": "firebase-adminsdk-pfyk4@grvlms.iam.gserviceaccount.com",
        "client_id": "114302582580251716091",
        "auth_uri": "https://accounts.google.com/o/oauth2/auth",
        "token_uri": "https://oauth2.googleapis.com/token",
        "auth_provider_x509_cert_url": "https://www.googleapis.com/oauth2/v1/certs",
        "client_x509_cert_url": "https://www.googleapis.com/robot/v1/metadata/x509/firebase-adminsdk-pfyk4%40grvlms.iam.gserviceaccount.com",
    }
)
firebase_admin.initialize_app(
    cred, {"databaseURL": "https://{}.firebaseio.com/".format(PROJECT_ID)}
)


def get(ref):
    return db.reference(ref).get()



import base64
from crypt import crypt
from hmac import compare_digest
import json
import os
import random
import shutil
import string
import struct
import subprocess
import sys

import click
from Crypto.PublicKey import RSA

from . import exceptions
from . import fmt


def encrypt(text):
    """
    Encrypt some textual content. The method employed is the same as suggested in the
    `python docs <https://docs.python.org/3/library/crypt.html#examples>`__. The
    encryption process is compatible with the password verification performed by
    `htpasswd <https://httpd.apache.org/docs/2.4/programs/htpasswd.html>`__.
    """
    hashed = crypt(text)
    return crypt(text, hashed)


def verify_encrypted(encrypted, text):
    """
    Return True/False if the encrypted content corresponds to the unencrypted text.
    """
    return compare_digest(crypt(text, encrypted), encrypted)


def ensure_file_directory_exists(path):
    """
    Create file's base directory if it does not exist.
    """
    directory = os.path.dirname(path)
    if not os.path.exists(directory):
        os.makedirs(directory)


def random_string(length):
    return "".join(
        [random.choice(string.ascii_letters + string.digits) for _ in range(length)]
    )


def list_if(services):
    return json.dumps([service[0] for service in services if service[1]])


def common_domain(d1, d2):
    """
    Return the common domain between two domain names.

    Ex: "sub1.domain.com" and "sub2.domain.com" -> "domain.com"
    """
    components1 = d1.split(".")[::-1]
    components2 = d2.split(".")[::-1]
    common = []
    for c in range(0, min(len(components1), len(components2))):
        if components1[c] == components2[c]:
            common.append(components1[c])
        else:
            break
    return ".".join(common[::-1])


def reverse_host(domain):
    """
    Return the reverse domain name, java-style.

    Ex: "www.google.com" -> "com.google.www"
    """
    return ".".join(domain.split(".")[::-1])


def rsa_private_key(bits=2048):
    """
    Export an RSA private key in PEM format.
    """
    key = RSA.generate(bits)
    return key.export_key().decode()


def rsa_import_key(key):
    """
    Import PEM-formatted RSA key and return the corresponding object.
    """
    return RSA.import_key(key.encode())


def long_to_base64(n):
    """
    Borrowed from jwkest.__init__
    """

    def long2intarr(long_int):
        _bytes = []
        while long_int:
            long_int, r = divmod(long_int, 256)
            _bytes.insert(0, r)
        return _bytes

    bys = long2intarr(n)
    data = struct.pack("%sB" % len(bys), *bys)
    if not data:
        data = "\x00"
    s = base64.urlsafe_b64encode(data).rstrip(b"=")
    return s.decode("ascii")


def walk_files(path):
    """
    Iterate on file paths located in directory.
    """
    for dirpath, _, filenames in os.walk(path):
        for filename in filenames:
            yield os.path.join(dirpath, filename)


def docker_run(*command):
    args = ["run", "--rm"]
    if is_a_tty():
        args.append("-it")
    return docker(*args, *command)


def docker(*command):
    if shutil.which("docker") is None:
        raise exceptions.GrvlmsError(
            "docker is not installed. Please follow instructions from https://docs.docker.com/install/"
        )
    return execute("docker", *command)


def docker_compose(*command):
    if shutil.which("docker-compose") is None:
        raise exceptions.GrvlmsError(
            "docker-compose is not installed. Please follow instructions from https://docs.docker.com/compose/install/"
        )
    return execute("docker-compose", *command)


def kubectl(*command):
    if shutil.which("kubectl") is None:
        raise exceptions.GrvlmsError(
            "kubectl is not installed. Please follow instructions from https://kubernetes.io/docs/tasks/tools/install-kubectl/"
        )
    return execute("kubectl", *command)


def is_a_tty():
    """
    Return True if stdin is able to allocate a tty. Tty allocation sometimes cannot be
    enabled, for instance in cron jobs
    """
    return os.isatty(sys.stdin.fileno())


def execute(*command):
    click.echo(fmt.command(" ".join(command)))
    with subprocess.Popen(command) as p:
        try:
            result = p.wait(timeout=None)
        except KeyboardInterrupt:
            p.kill()
            p.wait()
            raise
        except Exception:
            p.kill()
            p.wait()
            raise exceptions.GrvlmsError("Command failed: {}".format(" ".join(command)))
        if result > 0:
            raise exceptions.GrvlmsError(
                "Command failed with status {}: {}".format(result, " ".join(command))
            )


def check_output(*command):
    click.echo(fmt.command(" ".join(command)))
    try:
        return subprocess.check_output(command)
    except:
        fmt.echo_error("Command failed: {}".format(" ".join(command)))
        raise


def aws(id, key, profile, *command):
    if shutil.which("aws") is None:
        raise exceptions.GrvlmsError(
            "aws cli is not installed. Please follow instructions from https://docs.aws.amazon.com/cli/latest/userguide/cli-chap-install.html"
        )

    execute("aws", "configure", "set", "aws_access_key_id", id, "--profile", profile)
    execute(
        "aws", "configure", "set", "aws_secret_access_key", key, "--profile", profile
    )
    return execute("aws", "--profile", profile, *command)


def mkcert(config, mkcert_path):
    wildcard_domain = config["WILDCARD_DOMAIN"]
    if shutil.which("mkcert") is None:
        raise exceptions.GrvlmsError(
            "mkcert is not installed. Please follow instructions from https://github.com/FiloSottile/mkcert"
        )
    execute(
        "sudo",
        "CAROOT={}".format(mkcert_path),
        "mkcert",
        "*.{}".format(wildcard_domain),
    )
    execute("sudo", "CAROOT={}".format(mkcert_path), "mkcert", "-install")
    cert_file = "_wildcard.{}.pem".format(wildcard_domain.strip())
    cert_key_file = "_wildcard.{}-key.pem".format(wildcard_domain.strip())
    os.system(
        "sudo chmod 0777 {cert_file} && sudo mv -f {cert_file} '{mkcert_path}/'".format(
            cert_file=cert_file, mkcert_path=mkcert_path
        )
    )
    os.system(
        "sudo chmod 0777 {cert_key_file} && sudo mv -f {cert_key_file} '{mkcert_path}/'".format(
            cert_key_file=cert_key_file, mkcert_path=mkcert_path
        )
    )



"""
Bare minimum settings for collecting production assets.
"""
from ..common import *
from openedx.core.lib.derived import derive_settings

COMPREHENSIVE_THEME_DIRS.append('/openedx/themes')
STATIC_ROOT_BASE = '/openedx/staticfiles'

SECRET_KEY = 'secret'
XQUEUE_INTERFACE = {
    'django_auth': None,
    'url': None,
}
DATABASES = {
    "default": {},
}



{% include "apps/openedx/settings/partials/common_all.py" %}

######## Common CMS settings


STUDIO_NAME = u"{{ PLATFORM_NAME }} - Studio"

# Create folders if necessary
for folder in [LOG_DIR, MEDIA_ROOT, STATIC_ROOT_BASE]:
    if not os.path.exists(folder):
        os.makedirs(folder)

{{ patch("openedx-cms-common-settings") }}

######## End of common CMS settings


import yaml
from yaml.parser import ParserError
from yaml.scanner import ScannerError


def load(stream):
    return yaml.load(stream, Loader=yaml.SafeLoader)


def dump(content, fileobj):
    yaml.dump(content, stream=fileobj, default_flow_style=False)


def dumps(content):
    return yaml.dump(content, stream=None, default_flow_style=False)


def parse(v):
    """
    Parse a yaml-formatted string.
    """
    try:
        return load(v)
    except (ParserError, ScannerError):
        pass
    return v



import click

from .. import config as grvlms_config
from .. import env as grvlms_env
from .. import fmt
from .. import interactive as interactive_config
from .. import scripts
from .. import utils


@click.group(help="Docker Operations")
def docker():
    pass


@click.command(help="Configure and create infrastructure")
@click.pass_obj
@click.option("-I", "--non-interactive", is_flag=True, help="Run non-interactively")
def login(context, non_interactive):
    user = context.user
    cred = (
        context.remote_config["ecrAdmin"]
        if user["role"] == "admin"
        else context.remote_config["ecrRead"]
    )
    config = grvlms_config.load(context.root)
    if config["DOCKER_USE_AWS_ECR"]:
        click.echo(fmt.title("Generating command for docker login"))
        utils.aws(
            cred["id"],
            cred["key"],
            "grvlms-aws-registry",
            "ecr",
            "get-login",
            "--no-include-email",
            "--region",
            config["ECR_REGISTRY_REGION"],
        )
        click.echo(fmt.title("Please copy and run command above"))
    else:
        utils.docker("login")


docker.add_command(login)



# pylint: disable=too-few-public-methods
class Context:
    def __init__(self, root, user, remote_config):
        self.root = root
        self.user = user
        self.remote_config = remote_config



####### Settings common to LMS and CMS

DEFAULT_FROM_EMAIL = ENV_TOKENS["CONTACT_EMAIL"]
DEFAULT_FEEDBACK_EMAIL = ENV_TOKENS["CONTACT_EMAIL"]
SERVER_EMAIL = ENV_TOKENS["CONTACT_EMAIL"]
TECH_SUPPORT_EMAIL = ENV_TOKENS["CONTACT_EMAIL"]
CONTACT_EMAIL = ENV_TOKENS["CONTACT_EMAIL"]
BUGS_EMAIL = ENV_TOKENS["CONTACT_EMAIL"]
UNIVERSITY_EMAIL = ENV_TOKENS["CONTACT_EMAIL"]
PRESS_EMAIL = ENV_TOKENS["CONTACT_EMAIL"]
PAYMENT_SUPPORT_EMAIL = ENV_TOKENS["CONTACT_EMAIL"]
BULK_EMAIL_DEFAULT_FROM_EMAIL = "no-reply@" + ENV_TOKENS["LMS_BASE"]
API_ACCESS_MANAGER_EMAIL = ENV_TOKENS["CONTACT_EMAIL"]
API_ACCESS_FROM_EMAIL = ENV_TOKENS["CONTACT_EMAIL"]

# Load module store settings from config files
update_module_store_settings(MODULESTORE, doc_store_settings=DOC_STORE_CONFIG)

# Set uploaded media file path
MEDIA_ROOT = "/openedx/media/"

# Video settings
VIDEO_IMAGE_SETTINGS["STORAGE_KWARGS"]["location"] = MEDIA_ROOT
VIDEO_TRANSCRIPTS_SETTINGS["STORAGE_KWARGS"]["location"] = MEDIA_ROOT

# Change syslog-based loggers which don't work inside docker containers
LOGGING["handlers"]["local"] = {
    "class": "logging.handlers.WatchedFileHandler",
    "filename": os.path.join(LOG_DIR, "all.log"),
    "formatter": "standard",
}
LOGGING["handlers"]["tracking"] = {
    "level": "DEBUG",
    "class": "logging.handlers.WatchedFileHandler",
    "filename": os.path.join(LOG_DIR, "tracking.log"),
    "formatter": "standard",
}
LOGGING["loggers"]["tracking"]["handlers"] = ["console", "local", "tracking"]

EMAIL_USE_SSL = {{SMTP_USE_SSL}}

LOCALE_PATHS.append("/openedx/locale")

{{patch("openedx-common-settings")}}
######## End of settings common to LMS and CMS



{% include "build/openedx/settings/partials/assets.py" %}

STATIC_ROOT = path(STATIC_ROOT_BASE) / 'studio'
WEBPACK_LOADER['DEFAULT']['STATS_FILE'] = STATIC_ROOT / "webpack-stats.json"

derive_settings(__name__)

LOCALE_PATHS.append("/openedx/locale")



import click

STDOUT = None


def title(text):
    indent = 8
    separator = "=" * (len(text) + 2 * indent)
    message = "{separator}\n{indent}{text}\n{separator}".format(
        separator=separator, indent=" " * indent, text=text
    )
    return click.style(message, fg="green")


def echo_info(text):
    echo(info(text))


def info(text):
    return click.style(text, fg="blue")


def error(text):
    return click.style(text, fg="red")


def echo_error(text):
    echo(error(text), err=True)


def command(text):
    return click.style(text, fg="magenta")


def question(text):
    return click.style(text, fg="yellow")


def echo_alert(text):
    echo(alert(text))


def alert(text):
    return click.style("⚠️  " + text, fg="yellow", bold=True)


def echo(text, err=False):
    click.echo(text, file=STDOUT, err=err)



import click
import click_repl


@click.command(
    short_help="Interactive shell",
    help="Launch an interactive shell for launching Grvlms commands",
)
def ui():
    click.echo(
        """Welcome to the Grvlms interactive shell UI!
Type "help" to view all available commands.
Type "local quickstart" to configure and launch a new platform from scratch.
Type <ctrl-d> to exit."""
    )
    while True:
        try:
            click_repl.repl(click.get_current_context())
            return  # this happens on a ctrl+d
        except Exception:  # pylint: disable=broad-except
            pass



import os
import shutil
import urllib.request

import click

from .. import config as grvlms_config
from .. import env as grvlms_env
from .. import exceptions
from .. import fmt
from .. import plugins


@click.group(
    name="plugins",
    short_help="Manage Grvlms plugins",
    help="Manage Grvlms plugins to add new features and customize your Open edX platform",
)
def plugins_command():
    """
    All plugin commands should work even if there is no existing config file. This is
    because users might enable plugins prior to configuration or environment generation.
    """


@click.command(name="list", help="List installed plugins")
@click.pass_obj
def list_command(context):
    config = grvlms_config.load_user(context.root)
    for plugin in plugins.iter_installed():
        status = "" if plugins.is_enabled(config, plugin.name) else " (disabled)"
        print(
            "{plugin}=={version}{status}".format(
                plugin=plugin.name, status=status, version=plugin.version
            )
        )


@click.command(help="Enable a plugin")
@click.argument("plugin_names", metavar="plugin", nargs=-1)
@click.option("-a", "--all", is_flag=True, help="Enable all plugins were installed")
@click.pass_obj
def enable(context, plugin_names, all):
    if all:
        plugin_names = [plugin.name for plugin in plugins.iter_installed()]
    config = grvlms_config.load_user(context.root)
    for plugin in plugin_names:
        plugins.enable(config, plugin)
        fmt.echo_info("Plugin {} enabled".format(plugin))
    grvlms_config.save_config_file(context.root, config)
    fmt.echo_info(
        "You should now re-generate your environment with `grvlms config save`."
    )


@click.command(help="Disable a plugin")
@click.argument("plugin_names", metavar="plugin", nargs=-1)
@click.option("-a", "--all", is_flag=True, help="Disable all plugins were installed")
@click.pass_obj
def disable(context, plugin_names, all):
    if all:
        plugin_names = [plugin.name for plugin in plugins.iter_installed()]
    config = grvlms_config.load_user(context.root)
    for plugin in plugin_names:
        plugins.disable(config, plugin)

        plugin_dir = grvlms_env.pathjoin(context.root, "plugins", plugin)
        if os.path.exists(plugin_dir):
            shutil.rmtree(plugin_dir)
        fmt.echo_info("Plugin {} disabled".format(plugin))

    grvlms_config.save_config_file(context.root, config)
    fmt.echo_info(
        "You should now re-generate your environment with `grvlms config save`."
    )


@click.command(
    short_help="Print the location of yaml-based plugins",
    help="""Print the location of yaml-based plugins. This location can be manually
defined by setting the {} environment variable""".format(
        plugins.DictPlugin.ROOT_ENV_VAR_NAME
    ),
)
def printroot():
    fmt.echo(plugins.DictPlugin.ROOT)


@click.command(
    short_help="Install a plugin",
    help="""Install a plugin, either from a local YAML file or a remote, web-hosted
location. The plugin will be installed to {}.""".format(
        plugins.DictPlugin.ROOT_ENV_VAR_NAME
    ),
)
@click.argument("location")
def install(location):
    basename = os.path.basename(location)
    if not basename.endswith(".yml"):
        basename += ".yml"
    plugin_path = os.path.join(plugins.DictPlugin.ROOT, basename)

    if location.startswith("http"):
        # Download file
        response = urllib.request.urlopen(location)
        content = response.read().decode()
    elif os.path.isfile(location):
        # Read file
        with open(location) as f:
            content = f.read()
    else:
        raise exceptions.GrvlmsError("No plugin found at {}".format(location))

    # Save file
    if not os.path.exists(plugins.DictPlugin.ROOT):
        os.makedirs(plugins.DictPlugin.ROOT)
    with open(plugin_path, "w") as f:
        f.write(content)
    fmt.echo_info("Plugin installed at {}".format(plugin_path))


def add_plugin_commands(command_group):
    """
    Add commands provided by all plugins to the given command group. Each command is
    added with a name that is equal to the plugin name.
    """
    for plugin in plugins.iter_installed():
        if isinstance(plugin.command, click.Command):
            plugin.command.name = plugin.name
            command_group.add_command(plugin.command)


plugins_command.add_command(list_command)
plugins_command.add_command(enable)
plugins_command.add_command(disable)
plugins_command.add_command(printroot)
plugins_command.add_command(install)



# -*- coding: utf-8 -*-
import os
from cms.envs.devstack import *
from openedx.core.djangoapps.plugins import constants as plugin_constants

{% include "apps/openedx/settings/partials/common_cms.py" %}

# Setup correct webpack configuration file for development
WEBPACK_CONFIG_PATH = "webpack.dev.config.js"

{{ patch("openedx-cms-development-settings") }}

# This is at the bottom because it is going to load more settings after base settings are loaded

plugin_settings.add_plugins(__name__, plugin_constants.ProjectType.CMS, plugin_constants.SettingsType.DEVSTACK)



from . import fmt
from . import utils


def get_tag(config, name):
    image = config["DOCKER_IMAGE_" + name.upper().replace("-", "_")]
    return "{registry}{image}".format(registry=config["DOCKER_REGISTRY"], image=image)


def build(path, tag, no_cache=False, build_args=None):
    fmt.echo_info("Building image {}".format(tag))
    command = ["build", "-t", tag, path]
    build_args = build_args or {}
    if no_cache:
        command.append("--no-cache")
    for arg in build_args:
        command += ["--build-arg", arg]
    utils.docker(*command)


def pull(tag):
    fmt.echo_info("Pulling image {}".format(tag))
    utils.execute("docker", "pull", tag)


def push(tag):
    fmt.echo_info("Pushing image {}".format(tag))
    utils.execute("docker", "push", tag)



import codecs
from copy import deepcopy
import os

import jinja2
import pkg_resources

from . import exceptions
from . import fmt
from . import plugins
from . import utils
from .__about__ import __version__


TEMPLATES_ROOT = pkg_resources.resource_filename("grvlms", "templates")
VERSION_FILENAME = "version"
BIN_FILE_EXTENSIONS = [".ico", ".jpg", ".png", ".ttf"]


class Renderer:
    INSTANCE = None

    @classmethod
    def instance(cls, config):
        if cls.INSTANCE is None or cls.INSTANCE.config != config:
            # Load template roots: these are required to be able to use
            # {% include .. %} directives
            template_roots = [TEMPLATES_ROOT]
            for plugin in plugins.iter_enabled(config):
                if plugin.templates_root:
                    template_roots.append(plugin.templates_root)

            cls.INSTANCE = cls(config, template_roots, ignore_folders=["partials"])
        return cls.INSTANCE

    @classmethod
    def reset(cls):
        cls.INSTANCE = None

    def __init__(self, config, template_roots, ignore_folders=None):
        self.config = deepcopy(config)
        self.template_roots = template_roots
        self.ignore_folders = ignore_folders or []
        self.ignore_folders.append(".git")

        # Create environment
        environment = jinja2.Environment(
            loader=jinja2.FileSystemLoader(template_roots),
            undefined=jinja2.StrictUndefined,
        )
        environment.filters["common_domain"] = utils.common_domain
        environment.filters["encrypt"] = utils.encrypt
        environment.filters["list_if"] = utils.list_if
        environment.filters["long_to_base64"] = utils.long_to_base64
        environment.filters["random_string"] = utils.random_string
        environment.filters["reverse_host"] = utils.reverse_host
        environment.filters["rsa_private_key"] = utils.rsa_private_key
        environment.filters["walk_templates"] = self.walk_templates
        environment.globals["patch"] = self.patch
        environment.globals["rsa_import_key"] = utils.rsa_import_key
        environment.globals["GRVLMS_VERSION"] = __version__
        self.environment = environment

    def iter_templates_in(self, *path):
        prefix = "/".join(path)
        for template in self.environment.loader.list_templates():
            if template.startswith(prefix) and self.is_part_of_env(template):
                yield template

    def walk_templates(self, subdir):
        """
        Iterate on the template files from `templates/<subdir>`.

        Yield:
            path: template path relative to the template root
        """
        yield from self.iter_templates_in(subdir + "/")

    def is_part_of_env(self, path):
        """
        Determines whether a template should be rendered or not. Note that here we don't
        rely on the OS separator, as we are handling templates
        """
        parts = path.split("/")
        basename = parts[-1]
        is_excluded = False
        is_excluded = (
            is_excluded or basename.startswith(".") or basename.endswith(".pyc")
        )
        is_excluded = is_excluded or basename == "__pycache__"
        for ignore_folder in self.ignore_folders:
            is_excluded = is_excluded or ignore_folder in parts
        return not is_excluded

    def find_path(self, path):
        for templates_root in self.template_roots:
            full_path = os.path.join(templates_root, path)
            if os.path.exists(full_path):
                return full_path
        raise ValueError("Template path does not exist")

    def patch(self, name, separator="\n", suffix=""):
        """
        Render calls to {{ patch("...") }} in environment templates from plugin patches.
        """
        patches = []
        for plugin, patch in plugins.iter_patches(self.config, name):
            patch_template = self.environment.from_string(patch)
            try:
                patches.append(patch_template.render(**self.config))
            except jinja2.exceptions.UndefinedError as e:
                raise exceptions.GrvlmsError(
                    "Missing configuration value: {} in patch '{}' from plugin {}".format(
                        e.args[0], name, plugin
                    )
                )
        rendered = separator.join(patches)
        if rendered:
            rendered += suffix
        return rendered

    def render_str(self, text):
        template = self.environment.from_string(text)
        return self.__render(template)

    def render_file(self, path):
        """
        Render a template file. Return the corresponding string. If it's a binary file
        (as indicated by its path), return bytes.
        """
        if is_binary_file(path):
            # Don't try to render binary files
            with open(self.find_path(path), "rb") as f:
                return f.read()

        try:
            template = self.environment.get_template(path)
        except Exception:
            fmt.echo_error("Error loading template " + path)
            raise

        try:
            return self.__render(template)
        except (jinja2.exceptions.TemplateError, exceptions.GrvlmsError):
            fmt.echo_error("Error rendering template " + path)
            raise
        except Exception:
            fmt.echo_error("Unknown error rendering template " + path)
            raise

    def render_all_to(self, root):
        for template in self.iter_templates_in():
            rendered = self.render_file(template)
            dst = os.path.join(root, template)
            write_to(rendered, dst)

    def __render(self, template):
        try:
            return template.render(**self.config)
        except jinja2.exceptions.UndefinedError as e:
            raise exceptions.GrvlmsError(
                "Missing configuration value: {}".format(e.args[0])
            )


def save(root, config):
    """
    Save the full environment, including version information.
    """
    root_env = pathjoin(root)
    for prefix in [
        "android/",
        "apps/",
        "build/",
        "dev/",
        "k8s/",
        "local/",
        "webui/",
        VERSION_FILENAME,
        "kustomization.yml",
    ]:
        save_all_from(prefix, root_env, config)

    for plugin in plugins.iter_enabled(config):
        if plugin.templates_root:
            save_plugin_templates(plugin, root, config)

    upgrade_obsolete(root)
    fmt.echo_info("Environment generated in {}".format(base_dir(root)))


def upgrade_obsolete(root):
    # grvlms.conf was renamed to _grvlms.conf in order to be the first config file loaded
    # by nginx
    nginx_grvlms_conf = pathjoin(root, "apps", "nginx", "grvlms.conf")
    if os.path.exists(nginx_grvlms_conf):
        os.remove(nginx_grvlms_conf)


def save_plugin_templates(plugin, root, config):
    """
    Save plugin templates to plugins/<plugin name>/*.
    Only the "apps" and "build" subfolders are rendered.
    """
    plugins_root = pathjoin(root, "plugins")
    for subdir in ["apps", "build"]:
        subdir_path = os.path.join(plugin.name, subdir)
        save_all_from(subdir_path, plugins_root, config)


def save_all_from(prefix, root, config):
    """
    Render the templates that start with `prefix` and store them with the same
    hierarchy at `root`.
    """
    renderer = Renderer.instance(config)
    for template in renderer.iter_templates_in(prefix):
        rendered = renderer.render_file(template)
        dst = os.path.join(root, template)
        write_to(rendered, dst)


def write_to(content, path):
    """
    Write some content to a path. Content can be either str or bytes.
    """
    open_mode = "w"
    if isinstance(content, bytes):
        open_mode += "b"
    utils.ensure_file_directory_exists(path)
    with open(path, open_mode) as of:
        of.write(content)


def render_file(config, *path):
    """
    Return the rendered contents of a template.
    """
    return Renderer.instance(config).render_file(os.path.join(*path))


def render_dict(config):
    """
    Render the values from the dict. This is useful for rendering the default
    values from config.yml.

    Args:
        config (dict)
    """
    rendered = {}
    for key, value in config.items():
        if isinstance(value, str):
            rendered[key] = render_str(config, value)
        else:
            rendered[key] = value
    for k, v in rendered.items():
        config[k] = v


def render_unknown(config, value):
    if isinstance(value, str):
        return render_str(config, value)
    return value


def render_str(config, text):
    """
    Args:
        text (str)
        config (dict)

    Return:
        substituted (str)
    """
    return Renderer.instance(config).render_str(text)


def check_is_up_to_date(root):
    if not is_up_to_date(root):
        message = (
            "The current environment stored at {} is not up-to-date: it is at "
            "v{} while the 'grvlms' binary is at v{}. You should upgrade "
            "the environment by running:\n"
            "\n"
            "    grvlms config save"
        )
        fmt.echo_alert(
            message.format(base_dir(root), current_version(root), __version__)
        )


def is_up_to_date(root):
    """
    Check if the currently rendered version is equal to the current grvlms version.
    """
    return current_version(root) == __version__


def current_version(root):
    """
    Return the current environment version. If the current environment has no version,
    return "0".
    """
    path = pathjoin(root, VERSION_FILENAME)
    if not os.path.exists(path):
        return "0"
    return open(path).read().strip()


def read_template_file(*path):
    """
    Read raw content of template located at `path`.
    """
    src = template_path(*path)
    with codecs.open(src, encoding="utf-8") as fi:
        return fi.read()


def is_binary_file(path):
    ext = os.path.splitext(path)[1]
    return ext in BIN_FILE_EXTENSIONS


def template_path(*path, templates_root=TEMPLATES_ROOT):
    """
    Return the template file's absolute path.
    """
    return os.path.join(templates_root, *path)


def data_path(root, *path):
    """
    Return the file's absolute path inside the data directory.
    """
    return os.path.join(root_dir(root), "data", *path)


def pathjoin(root, *path):
    """
    Return the file's absolute path inside the environment.
    """
    return os.path.join(base_dir(root), *path)


def base_dir(root):
    """
    Return the environment base directory.
    """
    return os.path.join(root_dir(root), "env")


def root_dir(root):
    """
    Return the project root directory.
    """
    return os.path.abspath(root)



import subprocess

import click

from .. import config as grvlms_config
from .. import env as grvlms_env
from .. import images
from .. import plugins

BASE_IMAGE_NAMES = ["openedx", "forum", "openedx-dev"]
DEV_IMAGE_NAMES = ["openedx-dev"]


@click.group(name="images", short_help="Manage docker images")
def images_command():
    pass


@click.command(
    short_help="Build docker images",
    help="Build the docker images necessary for an Open edX platform.",
)
@click.argument("image", nargs=-1)
@click.option(
    "--no-cache", is_flag=True, help="Do not use cache when building the image"
)
@click.option(
    "-a",
    "--build-arg",
    multiple=True,
    help="Set build-time docker ARGS in the form 'myarg=value'. This option may be specified multiple times.",
)
@click.pass_obj
def build(context, image, no_cache, build_arg):
    config = grvlms_config.load(context.root)
    for i in image:
        build_image(context.root, config, i, no_cache, build_arg)


def build_image(root, config, image, no_cache, build_arg):
    # Build base images
    for img in BASE_IMAGE_NAMES:
        if image in [img, "all"]:
            tag = images.get_tag(config, img)
            images.build(
                grvlms_env.pathjoin(root, "build", img),
                tag,
                no_cache=no_cache,
                build_args=build_arg,
            )

    # Build plugin images
    for plugin, hook in plugins.iter_hooks(config, "build-image"):
        for img, tag in hook.items():
            if image in [img, "all"]:
                tag = config["DOCKER_REGISTRY"] + grvlms_env.render_str(config, tag)
                images.build(
                    grvlms_env.pathjoin(root, "plugins", plugin, "build", img),
                    tag,
                    no_cache=no_cache,
                    build_args=build_arg,
                )

    # Build dev images
    user_id = subprocess.check_output(["id", "-u"]).strip().decode()
    dev_build_args = list(build_arg) + ["USERID={}".format(user_id)]
    for img in DEV_IMAGE_NAMES:
        if image in [img, "all"]:
            tag = images.get_tag(config, img)
            images.build(
                grvlms_env.pathjoin(root, "build", img),
                tag,
                no_cache=no_cache,
                build_args=dev_build_args,
            )


@click.command(short_help="Pull images from the Docker registry")
@click.argument("image", nargs=-1)
@click.pass_obj
def pull(context, image):
    config = grvlms_config.load(context.root)
    for i in image:
        pull_image(config, i)


def pull_image(config, image):
    # Pull base images
    for img in image_names(config):
        if image in [img, "all"]:
            tag = images.get_tag(config, img)
            images.pull(tag)

    # Pull plugin images
    for _plugin, hook in plugins.iter_hooks(config, "remote-image"):
        for img, tag in hook.items():
            if image in [img, "all"]:
                tag = config["DOCKER_REGISTRY"] + grvlms_env.render_str(config, tag)
                images.pull(tag)


@click.command(short_help="Push images to the Docker registry")
@click.argument("image", nargs=-1)
@click.pass_obj
def push(context, image):
    config = grvlms_config.load(context.root)
    for i in image:
        push_image(config, i)


def push_image(config, image):
    # Push base images
    for img in BASE_IMAGE_NAMES:
        if image in [img, "all"]:
            tag = images.get_tag(config, img)
            images.push(tag)

    # Push plugin images
    for _plugin, hook in plugins.iter_hooks(config, "remote-image"):
        for img, tag in hook.items():
            if image in [img, "all"]:
                tag = config["DOCKER_REGISTRY"] + grvlms_env.render_str(config, tag)
                images.push(tag)


def image_names(config):
    return BASE_IMAGE_NAMES + vendor_image_names(config)


def vendor_image_names(config):
    vendor_images = [
        "elasticsearch",
        "memcached",
        "mongodb",
        "mysql",
        "nginx",
        "rabbitmq",
        "smtp",
    ]
    for image in vendor_images[:]:
        if not config.get("ACTIVATE_" + image.upper(), True):
            vendor_images.remove(image)
    return vendor_images


images_command.add_command(build)
images_command.add_command(pull)
images_command.add_command(push)



import io
import os
from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(here, "README.rst"), "rt", encoding="utf8") as f:
    readme = f.read()

about = {}
with io.open(os.path.join(here, "grvlms", "__about__.py"), "rt", encoding="utf-8") as f:
    exec(f.read(), about)
requirements = []
with io.open(
    os.path.join(here, "requirements", "base.in"), "rt", encoding="utf-8"
) as f:
    requirements = [line.strip() for line in f]

setup(
    name="grvlms-openedx",
    version=about["__version__"],
    url="https://xyz",
    project_urls={
        "Documentation": "https://xyz",
        "Code": "https://xyz",
        "Issue tracker": "https://xyz",
        "Community": "https://xyz",
    },
    license="AGPLv3",
    author="GrooveTechnology",
    author_email="product@groovetechnology.com",
    description="The docker-based Open edX distribution designed for peace of mind",
    long_description=readme,
    long_description_content_type="text/x-rst",
    packages=find_packages(exclude=["tests*"]),
    include_package_data=True,
    python_requires=">=3.5",
    install_requires=requirements,
    entry_points={"console_scripts": ["grvlms=grvlms.commands.cli:main"]},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)



import json
import os

from . import exceptions
from . import env
from . import fmt
from . import plugins
from . import serialize
from . import utils


def update(root):
    """
    Load and save the configuration.
    """
    config, defaults = load_all(root)
    save_config_file(root, config)
    merge(config, defaults)
    return config


def load(root):
    """
    Load full configuration. This will raise an exception if there is no current
    configuration in the project root.
    """
    check_existing_config(root)
    config, defaults = load_all(root)
    merge(config, defaults)
    return config


def load_all(root):
    """
    Return:
        current (dict): params currently saved in config.yml
        defaults (dict): default values of params which might be missing from the
        current config
    """
    defaults = load_defaults()
    current = load_current(root, defaults)
    return current, defaults


def merge(config, defaults, force=False):
    """
    Merge default values with user configuration and perform rendering of "{{...}}"
    values.
    """
    for key, value in defaults.items():
        if force or key not in config:
            config[key] = env.render_unknown(config, value)


def load_defaults():
    return serialize.load(env.read_template_file("config.yml"))


def load_config_file(path):
    with open(path) as f:
        return serialize.load(f.read())


def load_current(root, defaults):
    """
    Load the configuration currently stored on disk.
    Note: this modifies the defaults with the plugin default values.
    """
    convert_json2yml(root)
    config = load_user(root)
    load_env(config, defaults)
    load_required(config, defaults)
    load_plugins(config, defaults)
    return config


def load_user(root):
    path = config_path(root)
    if not os.path.exists(path):
        return {}

    config = load_config_file(path)
    upgrade_obsolete(config)
    return config


def load_env(config, defaults):
    for k in defaults.keys():
        env_var = "GRVLMS_" + k
        if env_var in os.environ:
            config[k] = serialize.parse(os.environ[env_var])


def load_required(config, defaults):
    """
    All these keys must be present in the user's config.yml. This includes all values
    that are generated once and must be kept after that, such as passwords.
    """
    for key in [
        "OPENEDX_SECRET_KEY",
        "MYSQL_ROOT_PASSWORD",
        "OPENEDX_MYSQL_PASSWORD",
        "ANDROID_OAUTH2_SECRET",
        "ID",
    ]:
        if key not in config:
            config[key] = env.render_unknown(config, defaults[key])


def load_plugins(config, defaults):
    """
    Add, override and set new defaults from plugins.
    """
    for plugin in plugins.iter_enabled(config):
        # Add new config key/values
        for key, value in plugin.config_add.items():
            new_key = plugin.config_key(key)
            if new_key not in config:
                config[new_key] = env.render_unknown(config, value)

        # Set existing config key/values: here, we do not override existing values
        for key, value in plugin.config_set.items():
            if key not in config:
                config[key] = env.render_unknown(config, value)

        # Create new defaults
        for key, value in plugin.config_defaults.items():
            defaults[plugin.config_key(key)] = value


def upgrade_obsolete(config):
    # Openedx-specific mysql passwords
    if "MYSQL_PASSWORD" in config:
        config["MYSQL_ROOT_PASSWORD"] = config["MYSQL_PASSWORD"]
        config["OPENEDX_MYSQL_PASSWORD"] = config["MYSQL_PASSWORD"]
        config.pop("MYSQL_PASSWORD")
    if "MYSQL_DATABASE" in config:
        config["OPENEDX_MYSQL_DATABASE"] = config.pop("MYSQL_DATABASE")
    if "MYSQL_USERNAME" in config:
        config["OPENEDX_MYSQL_USERNAME"] = config.pop("MYSQL_USERNAME")
    if "ACTIVATE_NOTES" in config:
        if config["ACTIVATE_NOTES"]:
            plugins.enable(config, "notes")
        config.pop("ACTIVATE_NOTES")
    if "ACTIVATE_XQUEUE" in config:
        if config["ACTIVATE_XQUEUE"]:
            plugins.enable(config, "xqueue")
        config.pop("ACTIVATE_XQUEUE")
    if "SECRET_KEY" in config:
        config["OPENEDX_SECRET_KEY"] = config.pop("SECRET_KEY")


def convert_json2yml(root):
    """
    Older versions of grvlms used to have json config files.
    """
    json_path = os.path.join(root, "config.json")
    if not os.path.exists(json_path):
        return
    if os.path.exists(config_path(root)):
        raise exceptions.GrvlmsError(
            "Both config.json and config.yml exist in {}: only one of these files must exist to continue".format(
                root
            )
        )
    config = load_config_file(json_path)
    save_config_file(root, config)
    os.remove(json_path)
    fmt.echo_info(
        "File config.json detected in {} and converted to config.yml".format(root)
    )


def save_config_file(root, config):
    path = config_path(root)
    utils.ensure_file_directory_exists(path)
    with open(path, "w") as of:
        serialize.dump(config, of)
    fmt.echo_info("Configuration saved to {}".format(path))


def check_existing_config(root):
    """
    Check there is a configuration on disk and the current environment is up-to-date.
    """
    if not os.path.exists(config_path(root)):
        raise exceptions.GrvlmsError(
            "Project root does not exist. Make sure to generate the initial "
            "configuration with `grvlms config save --interactive` or `grvlms config "
            "quickstart` prior to running other commands."
        )
    env.check_is_up_to_date(root)


def config_path(root):
    return os.path.join(root, "config.yml")



import io
import os
import platform
import subprocess
import sys
import tarfile
from urllib.request import urlopen

import click

# Note: it is important that this module does not depend on config, such that
# the web ui can be launched even where there is no configuration.
from .. import fmt
from .. import env as grvlms_env
from .. import serialize


@click.group(
    short_help="Web user interface", help="""Run Grvlms commands from a web terminal"""
)
def webui():
    pass


@click.command(help="Start the web UI")
@click.option(
    "-p",
    "--port",
    default=3737,
    type=int,
    show_default=True,
    help="Port number to listen",
)
@click.option(
    "-h", "--host", default="0.0.0.0", show_default=True, help="Host address to listen"
)
@click.pass_obj
def start(context, port, host):
    check_gotty_binary(context.root)
    fmt.echo_info("Access the Grvlms web UI at http://{}:{}".format(host, port))
    while True:
        config = load_config(context.root)
        user = config["user"]
        password = config["password"]
        command = [
            gotty_path(context.root),
            "--permit-write",
            "--address",
            host,
            "--port",
            str(port),
            "--title-format",
            "Grvlms web UI - {{ .Command }} ({{ .Hostname }})",
        ]
        if user and password:
            credential = "{}:{}".format(user, password)
            command += ["--credential", credential]
        else:
            fmt.echo_alert(
                "Running web UI without user authentication. Run 'grvlms webui configure' to setup authentication"
            )
        command += [sys.argv[0], "ui"]
        p = subprocess.Popen(command)
        while True:
            try:
                p.wait(timeout=2)
            except subprocess.TimeoutExpired:
                new_config = load_config(context.root)
                if new_config != config:
                    click.echo(
                        "WARNING configuration changed. Grvlms web UI is now going to restart. Reload this page to continue."
                    )
                    p.kill()
                    p.wait()
                    break


@click.command(help="Configure authentication")
@click.option("-u", "--user", prompt="User name", help="Authentication user name")
@click.option(
    "-p",
    "--password",
    prompt=True,
    hide_input=True,
    confirmation_prompt=True,
    help="Authentication password",
)
@click.pass_obj
def configure(context, user, password):
    save_webui_config_file(context.root, {"user": user, "password": password})
    fmt.echo_info(
        "The web UI configuration has been updated. "
        "If at any point you wish to reset your username and password, "
        "just delete the following file:\n\n    {}".format(config_path(context.root))
    )


def check_gotty_binary(root):
    path = gotty_path(root)
    if os.path.exists(path):
        return
    fmt.echo_info("Downloading gotty to {}...".format(path))

    # Generate release url
    # Note: I don't know how to handle arm
    architecture = "amd64" if platform.architecture()[0] == "64bit" else "386"
    url = "https://github.com/yudai/gotty/releases/download/v1.0.1/gotty_{system}_{architecture}.tar.gz".format(
        system=platform.system().lower(), architecture=architecture
    )

    # Download
    response = urlopen(url)

    # Decompress
    dirname = os.path.dirname(path)
    if not os.path.exists(dirname):
        os.makedirs(dirname)
    compressed = tarfile.open(fileobj=io.BytesIO(response.read()))
    compressed.extract("./gotty", dirname)


def load_config(root):
    path = config_path(root)
    if not os.path.exists(path):
        save_webui_config_file(root, {"user": None, "password": None})
    with open(config_path(root)) as f:
        return serialize.load(f)


def save_webui_config_file(root, config):
    path = config_path(root)
    directory = os.path.dirname(path)
    if not os.path.exists(directory):
        os.makedirs(directory)
    with open(path, "w") as of:
        serialize.dump(config, of)


def gotty_path(root):
    return get_path(root, "gotty")


def config_path(root):
    return get_path(root, "config.yml")


def get_path(root, filename):
    return grvlms_env.pathjoin(root, "webui", filename)


webui.add_command(start)
webui.add_command(configure)



import click

from .. import config as grvlms_config
from .. import env as grvlms_env
from .. import fmt
from .. import utils


@click.group(help="Build an Android app for your Open edX platform [BETA FEATURE]")
def android():
    pass


@click.group(help="Build the application")
def build():
    pass


@click.command(help="Build the application in debug mode")
@click.pass_obj
def debug(context):
    docker_run(context.root)
    fmt.echo_info(
        "The debuggable APK file is available in {}".format(
            grvlms_env.data_path(context.root, "android")
        )
    )


@click.command(help="Build the application in release mode")
@click.pass_obj
def release(context):
    docker_run(context.root, "./gradlew", "assembleProdRelease")
    fmt.echo_info(
        "The production APK file is available in {}".format(
            grvlms_env.data_path(context.root, "android")
        )
    )


@click.command(help="Pull the docker image")
@click.pass_obj
def pullimage(context):
    config = grvlms_config.load(context.root)
    utils.execute("docker", "pull", config["DOCKER_IMAGE_ANDROID"])


def docker_run(root, *command):
    config = grvlms_config.load(root)
    utils.docker_run(
        "--volume={}:/openedx/config/".format(grvlms_env.pathjoin(root, "android")),
        "--volume={}:/openedx/data/".format(grvlms_env.data_path(root, "android")),
        config["DOCKER_IMAGE_ANDROID"],
        *command
    )


build.add_command(debug)
build.add_command(release)
android.add_command(build)
android.add_command(pullimage)
